#	This file is part of stov, written by Helmut Pozimski in 2012.
#
#	stov is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, version 2 of the License.
#
#	stov is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with stov.  If not, see <http://www.gnu.org/licenses/>.


# -*- coding: utf8 -*-

import os
import gettext
import sys
import sqlite3
import subprocess


class conf(object):
	def __init__(self):
		"""Constructor

		Constructs the conf object with some reasonable default values which should
		work on most systems, existence of a local mail server is assumed.

		"""
		self.values = {
				"database": "stov.sqlite",
				"downloaddir": str(os.environ['HOME']) + "/stov",
				"maxvideos": "15",
				"mailhost": "localhost",
				"mailto": "root",
				"mailfrom": "stov@localhost",
				"smtpport": "25",
				"youtube-dl": "",
				"config_version": "2",
				"db_version": "1"
				}
		self.dbpath = str(os.environ['HOME']) + "/.stov/" + self.values["database"]

	def WriteConfig(self):
		"""Writes the configuration from the dictionary into the configuration file
		for stov. The configuration is written into the home directory of the user
		by default.

		"""
		try:
			self.configfile = open(str(os.environ['HOME']) + "/.stov/stov.config", "w")
		except IOError, os.error:
			print >> sys.stderr, _("Configuration could not be written, please"
					" check that the configuration directory exists and is writable")
		else:
			for key in self.values.iterkeys():
				self.configfile.write(key.upper() + "=" + self.values[key] + "\n")
			self.configfile.close()

	def CreateDB(self):
		"""Creates the database structure if it doesn't exist already."""
		try:
			self.__database = sqlite3.connect(self.dbpath)
		except sqlite3.OperationalError:
			print >> sys.stderr, _("The database could not be created, please "
				"check that the configuration directory exists and is writable")
		else:
			self.__dbcursor = self.__database.cursor()
			self.__dbcursor.execute("""CREATE TABLE subscriptions (
						id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
						title TEXT,
						name TEXT,
						type TEXT,
						searchstring TEXT,
						directory TEXT
						);""")
			self.__dbcursor.execute("""CREATE TABLE videos (
						id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
						title TEXT,
						description TEXT,
						ytid TEXT,
						subscription_id INTEGER,
						downloaded int
						);""")
			self.__database.commit()
			self.__database.close()

	def Initialize(self):
		"""Creates the necessary directory for the stov configuration and calls the
		internal methods to create the database and the configuration file.

		"""
		try:
			os.mkdir(str(os.environ['HOME']) + "/.stov", 0750)
		except os.error:
			print >> sys.stderr, _("Configuration directory could not be created, "
				"please check that your home directory exists and is writable")
		else:
			self.values["youtube-dl"] = subprocess.Popen(["which", "youtube-dl"],
				stdout=subprocess.PIPE).communicate()[0].strip()
			self.WriteConfig()
		self.CreateDB()

	def ReadConfig(self):
		"""Reads the existing configuration file and places the values in the
		dictionary. Existing values (such as default values) are overwritten.

		"""
		try:
			self.configfile = open(str(os.environ['HOME']) + "/.stov/stov.config", "r")
		except IOError:
			print >> sys.stderr, _("Configuration could not be "
				"read, please check that the configuration file exists and is readable")
		for lines in self.configfile:
			self.tmpline = lines.strip()
			self.tmplist = self.tmpline.split("=")
			self.values[self.tmplist[0].lower()] = self.tmplist[1]
		self.configfile.close()
		self.dbpath = str(os.environ['HOME']) + "/.stov/" + self.values["database"]

	def CheckConfig(self):
		"""Checks if the configuration is up-to-date with the running
		stov version.

		"""
		try:
			self.__currentversion = int(self.values["config_version"])
		except ValueError:
			print >> sys.stderr, _("Invalid config version read")
		self.values["config_version"] = "0"
		self.ReadConfig()
		if self.values["config_version"] == "0" \
		or int(self.values["config_version"]) < self.__currentversion:
			self.values["config_version"] = str(self.__currentversion)
			return False
		else:
			self.values["config_version"] = str(self.__currentversion)
			return True

	def UpdateConfig(self):
		"""Update the configuration to the latest version"""
		self.ReadConfig()
		self.WriteConfig()

	def CheckDB(self):
		"""Checks the database if it is up-to-date"""
		try:
			self.__currentversion = int(self.values["db_version"])
		except ValueError:
			print >> sys.stderr, _("Invalid config version read")
		self.values["db_version"] = "0"
		self.ReadConfig()
		if self.values["db_version"] == "0" \
		or int(self.values["db_version"]) < self.__currentversion:
			self.values["db_version"] = str(self.__currentversion)
			return False
		else:
			self.values["db_version"] = str(self.__currentversion)
			return True

	def UpdateDB(self):
		"""Performs database changes that need to be done"""
		if int(self.values["db_version"]) == 1:
			pass
