#! /usr/bin/env python
# -*- coding: utf8 -*-

#stov - a program to subscribe to channels and users from online video
#	portals and download the videos automatically
#
#	written by Helmut Pozimski in 2012
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; version 2
#	of the License.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor,
#	Boston, MA 02110-1301, USA.

import sys
import gettext
import os
import sqlite3
import email
import smtplib
import subprocess
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from optparse import OptionParser

import subscription
import youtube
import configuration

"""Determine the path where the stov files are for localization"""

locale_path = sys.path[0] + "/locale"

"""Initialize gettext to support translation of the program"""

try:
	trans = gettext.translation("stov", locale_path, ["de"])
except IOError:
	gettext.install("stov")
	print >> sys.stderr, _("Translation files could not be found, localization "
				"won't be available")
else:
	trans.install()

"""Overwrite the default OptionParser class so error messages
can be localized

"""


class MyOptionParser(OptionParser):
	def error(self, msg):
		if "invalid integer" in msg:
			print >> sys.stderr, _("option %s requires an integer value"
						% msg.split()[1])
			self.exit()
		elif "an argument" in msg:
			print >> sys.stderr, _("option %s requires an argument" % msg.split()[0])
			self.exit()
		elif "no such" in msg:
			print >> sys.stderr, _("invalid option %s" % msg.split()[3])
			self.exit()
		else:
			print >> sys.stderr, msg
			self.exit()

"""Process the given options and parameters,
		add: Add a new subscription (which can be a search, channel or playlist)
		channel: with add, specify the name of the channel or user
		lssubs: List the currently available subscriptions
		remove: remove a subscription
		update: update the information about the available videos
		download: download all available videos which haven't already been downloaded
		search: optionally add a search string to a new subscription or create a
			search subscription with add
		playlist: with add, subscribe to a youtube playlist
		catchup: Mark all videos in a subscription as downloaded
		version: Print version number
"""

parser = MyOptionParser(usage=_("Usage: %prog [options]"), prog="stov",
				add_help_option=True, conflict_handler="resolve")

parser.add_option("-a", "--add", dest="add", action="store_true",
help=_("Add a new subscription (requires either --search, \
--channel or --playlist)"))
parser.add_option("-p", "--playlist", dest="playlist",
help=_("Add a new Playlist subscription (requires add)"))
parser.add_option("-l", "--lssubs", action="store_true", dest="list",
help=_("List the currently available subscriptions"))
parser.add_option("-r", "--remove", type="int", dest="deleteid",
help=_("remove a subscription"))
parser.add_option("-u", "--update", action="store_true", dest="update",
help=_("update the information about the available videos"))
parser.add_option("-d", "--download", action="store_true", dest="download",
help=_("download all available videos which haven't already been downloaded"))
parser.add_option("-s", "--search", dest="searchparameter",
help=_("optionally add a search string to a new channel subscription or \
create a new search subscription (requires --add)"))
parser.add_option("-l", "--lsvids", type="int", dest="subscriptionid",
help=_("Print all videos from a subscription"))
parser.add_option("-c", "--catchup", dest="catchup",
help=_("Mark all videos from one channel as read \
(requires subscription-id as argument)"))
parser.add_option("-c", "--channel", dest="channel",
help=_("specify a channel for a new subscription (requires --add)"))
parser.add_option("-l", "--license", dest="license", action="store_true",
help=_("show the license of the program"))
parser.add_option("-v", "--version", dest="version", action="store_true",
help=_("show the current running version number"))
(options, arguments) = parser.parse_args()

"""Check if stov is run directly from command line since it shouldn't be
loaded as a module

"""

if __name__ != "__main__":
	print >> sys.stderr, _("This file should not be imported as a module "
				"please run it directly from command line")
	sys.exit(1)


"""Create the lock file which is used to determine if another instance is
already running by chance, the program shouldn't be run in this case since
we want to prevent concurent access to the database.

"""

if os.access("/tmp/stov.lock", os.F_OK):
	try:
		lockfile = open("/tmp/stov.lock", "r")
	except IOError:
		print >> sys.stderr, _("Lock file could not be opened, please check that "
					"it exists and is readable, quitting now")
		sys.exit(1)
	oldpid = lockfile.read().strip()
	if os.access("/proc/" + oldpid, os.F_OK):
		print >> sys.stderr, _("The lock file already exists, probably another"
					"instance of this program is already running\n"
					"if you are sure this is not the case, delete it"
					" manually and try again!")
		sys.exit(1)
	lockfile.close()
	if os.access("/proc/" + oldpid, os.F_OK) is not True:
		try:
			os.remove("/tmp/stov.lock")
		except os.error:
			print >> sys.stderr, _("Old lock file could not be deleted!")
try:
	lockfile = open("/tmp/stov.lock", "w")
	lockfile.write(str(os.getpid()))
	lockfile.close()
except IOError:
	print _("Lock file could not be created, please check that /tmp is "
		"writable and properly configured, quitting now")
	sys.exit(1)

"""Check if the configuration directory exists and is writeable. If it \
doesnt, create it using the configuration class.

"""
if os.access(os.environ['HOME'] + "/.stov", os.F_OK & os.W_OK) is not True:
	print _("This seems to be the first time you run the programm, it will now "
		"create the default configuration for you")
	conf = configuration.conf()
	conf.Initialize()
else:
	conf = configuration.conf()
	if conf.CheckConfig() is not True:
		print _("Your configuration needs to be updated, performing"
			" update now.")
		conf.UpdateConfig()
	if conf.CheckDB() is not True:
		print _("Your database needs to be updated, performing"
			" update now.")
		conf.UpdateDB()
	conf.ReadConfig()

"""youtube-dl is really a dependency but the program will run with limited\
functionality without it so we need to check that here

"""

if conf.values["youtube-dl"] == "":
	conf.values["youtube-dl"] = subprocess.Popen(["which", "youtube-dl"],
				stdout=subprocess.PIPE).communicate()[0].strip()
	if os.access(conf.values["youtube-dl"], os.F_OK & os.R_OK & os.X_OK):
		print _("Found youtube-dl, writing to configuration file.")
		conf.WriteConfig()
	else:
		print >> sys.stderr, _("Could not find youtube-dl, it either does not exist, "
					"is not readable or not executable. Please note that "
					"youtube-dl is not needed for the program to run but is"
					" needed to use the download option which won't work otherwise."
					" If youtube-dl isn't found automatically, you may also enter "
					"the path to it in the configuration file.")

"""Variable to save the text that is later sent as e-mail"""
MailContent = []

"""Check which options are given on the command line and
run the corresponding code

"""
if options.add is True:
	AddSub = True
	if options.channel is not None and options.searchparameter is None:
		NewSubscription = subscription.sub(type="channel",
		name=options.channel, conf=conf)
	elif options.channel is not None and options.searchparameter is not None:
		NewSubscription = subscription.sub(type="channel",
		name=options.channel, search=options.searchparameter, conf=conf)
	elif options.channel is None and options.searchparameter is not None:
		NewSubscription = subscription.sub(type="search",
		name=_("Search_"), search=options.searchparameter, conf=conf)
	elif options.add is True and options.playlist is not None:
		if options.searchparameter is not None:
			print _("Playlists do not support searching, search option will "
				"be ignored!")
		NewSubscription = subscription.sub(type="playlist",
				name=options.playlist, conf=conf)
		NewSubscription.AddSub()
		NewSubscription.UpdateVideos()
	else:
		print >> sys.stderr, _("No valid subscription options given, "
					"subscription could not be added")
		AddSub = False
	if AddSub is True:
		NewSubscription.AddSub()
		NewSubscription.UpdateVideos()
		print _("New subscription ") + NewSubscription.GetTitle()\
		+ _(" successfully added")
elif options.list is True:
	try:
		database = sqlite3.connect(conf.dbpath)
		cursor = database.cursor()
	except sqlite3.OperationalError:
		print _("Could not access the database, please check path "
			"and permissions and try again!")
	else:
		cursor.execute("SELECT id, title FROM subscriptions")
		Listofsubscriptions = cursor.fetchall()

	if len(Listofsubscriptions) != 0:
		print _("ID Title")
		for subscription in Listofsubscriptions:
			if subscription[0] is not None:
				print str(subscription[0]) + " " + subscription[1]
	else:
		print _("No subscriptions added yet, add one!")
	database.close()
elif options.deleteid is not None:
	try:
		DeleteId = int(options.deleteid)
		Subscription = subscription.sub(type="", name="delete",
		id=DeleteId, conf=conf)
	except ValueError:
		print _("Invalid Option, please use the ID of the subscription"
			"you want to delete as parameter for the remove option")
	Subscription.Delete()
elif options.update is True:
	listofsubscriptions = []
	try:
		database = sqlite3.connect(conf.dbpath)
		cursor = database.cursor()
	except sqlite3.OperationalError:
		print _("Could not access the database, please check path and "
			"permissions and try again!")
	else:
		cursor.execute("SELECT id,title,type,name,searchstring,directory \
		FROM subscriptions")
		subscriptions = cursor.fetchall()
		database.close()
	for element in subscriptions:
		listofsubscriptions.append(subscription.sub(id=element[0],
		title=element[1], type=element[2], name=element[3],
		search=element[4], directory=element[5], conf=conf))
	for element in listofsubscriptions:
		element.UpdateVideos()
elif options.download is True:
	listofsubscriptions = []
	try:
		database = sqlite3.connect(conf.dbpath)
		cursor = database.cursor()
	except sqlite3.OperationalError:
		print _("Could not access the database, please check path"
			"and permissions and try again!")
	else:
		cursor.execute("SELECT id,title,type,name,searchstring,directory \
FROM subscriptions")
		subscriptions = cursor.fetchall()

	for element in subscriptions:
		listofsubscriptions.append(subscription.sub(id=element[0],
		title=element[1], type=element[2], name=element[3],
		search=element[4], directory=element[5], conf=conf))
	VideosDownloaded = 0
	for element in listofsubscriptions:
		element.GetVideos()
		element.DownloadVideos()
		for entry in element.DownloadedVideos:
			MailContent.append(entry)
		videosdownloaded = len(MailContent)
	if videosdownloaded > 0:
		MailText = u""
		msg = MIMEMultipart()
		if videosdownloaded == 1:
			msg["Subject"] = _("Downloaded %i new video") % videosdownloaded
			MailText = _("The following episode has been downloaded by stov: \n\n")
		else:
			msg["Subject"] = _("Downloaded %i new videos") % videosdownloaded
			MailText = _("The following episodes have been downloaded by stov: \n\n")
		msg["From"] = "stov <%s>" % conf.values["mailfrom"]
		msg["To"] = "<%s>" % conf.values["mailto"]
		for line in MailContent:
			MailText += line + "\n"
		msgtext = MIMEText(MailText.encode("utf8"), _charset="utf8")
		msg.attach(msgtext)
		Mail = smtplib.SMTP(conf.values["mailhost"], conf.values["smtpport"])
		try:
			Mail = smtplib.SMTP(conf.values["mailhost"], conf.values["smtpport"])
		except SMTPHeloError:
			_("Could not connect to the smtp server, please check your settings!")
		Mail.sendmail(conf.values["mailfrom"], conf.values["mailto"],
		msg.as_string())
		Mail.quit()

elif options.subscriptionid is not None:
	try:
		database = sqlite3.connect(conf.dbpath)
		cursor = database.cursor()
	except sqlite3.OperationalError:
		print >> sys.stderr, _("Could not access the database, please check"
					"path and permissions and try again!")
	else:
		subscription_video_select = "SELECT id,title,type,name,searchstring,\
directory FROM subscriptions where id=?"
		cursor.execute(subscription_video_select, (options.subscriptionid,))
		Data = cursor.fetchall()

	if Data != []:
		Subscription = subscription.sub(id=Data[0][0], title=Data[0][1],
		type=Data[0][2], name=Data[0][3], search=Data[0][4],
		directory=Data[0][5], conf=conf)
		Subscription.GetVideos()
		Subscription.PrintVideos()
	else:
		print _("Invalid subscription, please check the list and try again")
elif options.catchup is not None:
	try:
		database = sqlite3.connect(conf.dbpath)
		cursor = database.cursor()
	except sqlite3.OperationalError:
		print >> sys.stderr, _("Could not access the database, please check "
					"path and permissions and try again!")
	else:
		subscriptionselect = "SELECT title from subscriptions where id=?"
		cursor.execute(subscriptionselect, (options.catchup,))
		sub_data = cursor.fetchall()
		if sub_data != []:
			catchup_sql = "UPDATE videos SET downloaded = 1 WHERE subscription_id =?"
			cursor.execute(catchup_sql, (options.catchup,))
			database.commit()
			database.close()
		else:
			print >> sys.stderr, _("Subscription could not be updated, "
						"please check if the ID given is correct")


elif options.license is True:
	print """
	stov is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	stov is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with stov.  If not, see <http://www.gnu.org/licenses/>.
	"""
elif options.version is True:
	print "0.2"
else:
	parser.print_help()

"""Remove the lock file and end the program so it can be run again"""

try:
	os.remove("/tmp/stov.lock")
	sys.exit(0)
except os.error:
	print >> sys.stderr, _("Could not delete the lock file. Please check what "
				"went wrong and clean up manually!")
